import junit.framework.*;

public class BowlingGameTest extends TestCase
{
    private BowlingGame game;

    protected void setUp()
    {
        game = new BowlingGame();
    }

    public void testAllGutters()
    {
        for (int i = 0; i < 20; i++)
            game.roll( 0 );
        assertEquals( "all gutter balls gives a score of 0", 0, game.score() );
    }

    public void testAllOpenFrames()
    {
        for (int i = 0; i < 10; i++)
        {
            game.roll( 5 );
            game.roll( 4 );
        }
        assertEquals( "ten frames of 9 give a score of 90", 90, game.score() );
    }

    public void testSpare()
    {
        game.roll( 4 );
        game.roll( 6 );
        game.roll( 5 );
        game.roll( 4 );
        for (int i = 0; i < 8; i++)
        {
            game.roll( 0 );
            game.roll( 0 );
        }
        assertEquals( "spare counts next ball, too", 24, game.score() );
    }

    public void testStrike()
    {
        game.roll( 10 );
        game.roll( 5 );
        game.roll( 4 );
        game.roll( 3 );
        game.roll( 0 );
        for (int i = 0; i < 7; i++)
        {
            game.roll( 0 );
            game.roll( 0 );
        }
        assertEquals( "first frame is worth 19", 31, game.score() );
    }

    public void testPerfectGame()
    {
        for (int i = 0; i < 12; i++)
            game.roll( 10 );
        assertEquals( "perfect game", 300, game.score() );
    }

    public void testAlternatingStrikesAndSpares()
    {
        for (int i = 0; i < 5; i++)
        {
            game.roll( 10 );
            game.roll( 7 );
            game.roll( 3 );
        }
        game.roll( 10 );
        assertEquals( "alternating game", 200, game.score() );
    }

    public void testScoreSheetStrings()
    {
        for (int i = 0; i < 3; i++)
        {
            game.roll( 10 );
            game.roll( 7 );
            game.roll( 3 );
            game.roll( 2 );
            game.roll( 4 );
        }
        game.roll( 10 );
        game.roll( 10 );
        game.roll( 7 );

        String ballString  = "XXX|7 /|2 4|XXX|7 /|2 4|XXX|7 /|2 4|X X 7|";
        String totalString = " 20| 32| 38| 58| 70| 76| 96|108|114|141|";
        String fullString  = ballString + "\n" + totalString;

        assertTrue( "full game print String",
                    fullString.equals( game.toString() ) );
    }

    public static Test suite()
    {
        return new TestSuite( BowlingGameTest.class );
    }

    public static void main( String[] args )
    {
        junit.textui.TestRunner.run( suite() );
    }
}
