import junit.framework.*;

public class FramePrintTest extends TestCase
{
    private Frame game;

    protected void setUp()
    {
        game = new Frame();
        setUpNineFrames();
    }

    public void testOneOpenFrameBallString()
    {
        Frame frame = new Frame();
        frame.roll( 7 );
        frame.roll( 2 );
        assertTrue( "7 2|".equals(frame.ballString()) );
    }

    public void testOneSpareFrameBallString()
    {
        Frame frame = new Frame();
        frame.roll( 4 );
        frame.roll( 6 );
        assertTrue( "4 /|".equals(frame.ballString()) );
    }

    public void testOneStrikeFrameBallString()
    {
        Frame frame = new Frame();
        frame.roll( 10 );
        assertTrue( "XXX|".equals(frame.ballString()) );
    }

    public void testFullGameBallString()
    {
        game.roll( 6 );
        game.roll( 3 );

        String expectedAnswer = "7 2|8 /|4 /|7 2|6 /|XXX|9 /|7 1|5 3|6 3|";
        assertTrue( expectedAnswer.equals( game.ballString() ) );
    }

    public void testFullGameEndsWithSpareBallString()
    {
        game.roll( 6 );
        game.roll( 4 );
        game.roll( 7 );

        String expectedAnswer = "7 2|8 /|4 /|7 2|6 /|XXX|9 /|7 1|5 3|6 / 7|";
        assertTrue( expectedAnswer.equals( game.ballString() ) );
    }

    public void testFullGameEndsWithSpareAndStrikeBallString()
    {
        game.roll( 6 );
        game.roll( 4 );
        game.roll( 10 );

        String expectedAnswer = "7 2|8 /|4 /|7 2|6 /|XXX|9 /|7 1|5 3|6 / X|";
        assertTrue( expectedAnswer.equals( game.ballString() ) );
    }

    public void testFullGameEndsWithStrikeBallString()
    {
        game.roll( 10 );
        game.roll( 4 );
        game.roll( 5 );

        String expectedAnswer = "7 2|8 /|4 /|7 2|6 /|XXX|9 /|7 1|5 3|X 4 5|";
        assertTrue( expectedAnswer.equals( game.ballString() ) );
    }

    public void testFullGameEndsWithStrikeAndStrikeBonusesBallString()
    {
        game.roll( 10 );
        game.roll( 10 );
        game.roll( 10 );

        String expectedAnswer = "7 2|8 /|4 /|7 2|6 /|XXX|9 /|7 1|5 3|X X X|";
        assertTrue( expectedAnswer.equals( game.ballString() ) );
    }

    public void testOneOpenFrameOneDigitTotalString()
    {
        Frame frame = new Frame();
        frame.roll( 4 );
        frame.roll( 3 );
        assertTrue( "  7|".equals(frame.totalString()) );
    }

    public void testOpenFrameTwoDigitsTotalString()
    {
        Frame frame = new Frame();
        frame.roll( 4 );
        frame.roll( 3 );
        frame.roll( 8 );
        frame.roll( 1 );
        assertTrue( "  7| 16|".equals(frame.totalString()) );
    }

    public void testStrikeTotalString()
    {
        Frame strikeFrame = new Frame();
        strikeFrame.roll( 10 );
        strikeFrame.roll( 10 );
        strikeFrame.roll( 2 );
        strikeFrame.roll( 3 );
        assertTrue( " 22| 37| 42|".equals( strikeFrame.totalString() ) );
    }

    public void testFullGameTotalString()
    {
        game.roll( 6 );
        game.roll( 3 );

        String expectedAnswer = "  9| 23| 40| 49| 69| 89|106|114|122|131|";
        assertTrue( expectedAnswer.equals( game.totalString() ) );
    }

    public void testFullGameEndsWithSpareTotalString()
    {
        game.roll( 6 );
        game.roll( 4 );
        game.roll( 7 );

        String expectedAnswer = "  9| 23| 40| 49| 69| 89|106|114|122|139|";
        assertTrue( expectedAnswer.equals( game.totalString() ) );
    }

    public void testFullGameEndsWithStrikeTotalString()
    {
        game.roll( 10 );
        game.roll( 10 );
        game.roll( 2 );

        String expectedAnswer = "  9| 23| 40| 49| 69| 89|106|114|122|144|";
        assertTrue( expectedAnswer.equals( game.totalString() ) );
    }

    public void testFullGameEndsWithStrikeAndStrikeBonusesTotalString()
    {
        game.roll( 10 );
        game.roll( 10 );
        game.roll( 10 );

        String expectedAnswer = "  9| 23| 40| 49| 69| 89|106|114|122|152|";
        assertTrue( expectedAnswer.equals( game.totalString() ) );
    }

    public static Test suite()
    {
        return new TestSuite( FramePrintTest.class );
    }

    public static void main( String[] args )
    {
        junit.textui.TestRunner.run( suite() );
    }

    private void setUpNineFrames()
    {
        game.roll( 7 );
        game.roll( 2 );
        game.roll( 8 );
        game.roll( 2 );
        game.roll( 4 );
        game.roll( 6 );
        game.roll( 7 );
        game.roll( 2 );
        game.roll( 6 );
        game.roll( 4 );
        game.roll( 10 );
        game.roll( 9 );
        game.roll( 1 );
        game.roll( 7 );
        game.roll( 1 );
        game.roll( 5 );
        game.roll( 3 );
    }
}
